###################################################################################################

#	LONG LINES AT POLLING STATIONS?
#	OBSERVATIONS FROM AN ELECTION DAY FIELD STUDY

#  Douglas M. Spencer and Zachary S. Markovits

#  Election Law Journal, Vol. 9, No. 1, pp. 3-17 (2010)

#	Replication code by Doug Spencer (dspencer@berkeley.edu)
#	Last updated: December 16, 2012 (R 2.15.2 on Mac OSX 10.6.8)

###################################################################################################

# Library in required packages

library(foreign)
library(sandwich)
library(lmtest)
library(ggplot2) # requires 'reshape', 'plyr', 'grid', and 'proto' packages

##########

# Set working directory to folder with data files
#setwd("~/Desktop/Lines/")

##########

# Read in data...

# ...from Attendance Counters (ACs)
x <- read.dta("http://www.dougspencer.org/data/ACdata.dta", convert.underscore=T, convert.factors=F)

# ...from Voter Observers (VOs)
y <- read.dta("http://www.dougspencer.org/data/VOdata.dta", convert.underscore=T, convert.factors=F)

###################################################################################################
#	FIGURES
###################################################################################################

# FIGURE 3 - Distribution of arrivals

arrive <- aggregate(arriv~time.numeric+county, data=x, FUN=sum)

fig.3 <- qplot(time.numeric, arriv, data=arrive, linetype=factor(county), geom="line", main="Voter Arrivals By County") +
	scale_y_continuous("Number of arrivals", limits=c(0,105)) +
	scale_x_continuous("Time of day", limits=c(7,20), breaks=c(7:20), labels=c("7a", "8", "9", "10", "11", "12p", "1", "2", "3", "4", "5", "6", "7", "8")) +
	scale_linetype_manual("CA county", values=c("Alameda"=1, "Napa"=2, "San Mateo"=3)) +
	theme(panel.background=element_rect(fill = "white", colour = "black")) +
	theme(axis.text.x=element_text(size=9, colour="grey35"), axis.text.y=element_text(size=9, colour="grey35")) +
	theme(panel.grid.minor = element_blank(), panel.grid.major = element_line("gray", size = 0.2))

#pdf("Arrivals.pdf", width=6.5, height=4)
print(fig.3)
#dev.off()

### And two subplots that look at arrival distributions by polling station:

arrive.ps <- aggregate(arriv~time.numeric+county+pollst, data=x, FUN=sum)

p.pollst.1 <- qplot(time.numeric, arriv, data=arrive.ps, facets=county~pollst, geom="line") +
	scale_y_continuous("Number of arrivals", limits=c(0,15)) +
	scale_x_continuous("Time of day", limits=c(7,20), breaks=c(7:20)) +
	theme(panel.background=element_rect(fill = "white", colour = "black")) +
	theme(axis.text.x=element_text(size=5), axis.text.y=element_text(size=7))

#pdf("ArriveByPollSt.pdf", width=12, height=5.5)
print(p.pollst.1)
#dev.off()

p.pollst.2 <- qplot(time.numeric, arriv, data=arrive.ps, facets=county~pollst, geom="smooth", se=F) +
	scale_y_continuous("Number of arrivals", limits=c(0,15)) +
	scale_x_continuous("Time of day", limits=c(7,20), breaks=c(7:20)) +
	theme(panel.background=element_rect(fill = "white", colour = "black")) +
	theme(axis.text.x=element_text(size=5), axis.text.y=element_text(size=7))

#pdf("ArriveByPollSt-Loess.pdf", width=12, height=5.5)
print(p.pollst.2)
#dev.off()

##########

#	FIGURE 4 - Aggregate length of lines

lines <- aggregate(line~time.numeric, data=x, FUN=sum)

fig.4 <- qplot(time.numeric, line, data=lines, geom="line") +
	scale_y_continuous("Number of people standing in line", limits=c(0,75)) +
	scale_x_continuous("Time of day", limits=c(7,20), breaks=c(7:20)) +
	theme(panel.background=element_rect(fill = "white", colour = "black")) +
	theme(axis.text.x=element_text(size=11), axis.text.y=element_text(size=11))

#pdf("LineLength.pdf", width=9, height=6)
print(fig.4)
#dev.off()

##########

#	FIGURE 5 - Probability that voters reneged

x$line.5 <- 0
x$line.5[x$line > 0 & x$line < 5] <- 1
x$line.5[x$line >= 5] <- 2

renege <- data.frame(
	line=c(0:2), 
	renege=c(mean(x$renege[x$line.5==0], na.rm=T),
			mean(x$renege[x$line.5==1], na.rm=T),
			mean(x$renege[x$line.5==2], na.rm=T)
))

#pdf("Renege.pdf", width=8, height=6)

plot(renege$line, renege$renege, type="n", axes=F, xlim=c(-.1,2.1), ylim=c(0,0.5),
		xlab="Number of people standing in line", ylab="Probability of reneging")
axis(1, at=c(0,1,2), labels=c("Line = 0", "0 < x <= 5", "5 < x"))
axis(2, at=seq(0,0.5, by=0.1))
segments(x0=renege[1,1], y0=renege[1,2], x1=renege[2,1], y1=renege[2,2], lty=1)
segments(x0=renege[2,1], y0=renege[2,2], x1=renege[3,1], y1=renege[3,2], lty=2)
points(renege[1:2,], pch=19)
points(renege[3,], pch=21, bg="white")
text(0, 0.085, "n = 1766")
text(0.01, 0.03, paste(round(renege[1,2], digits=3)), font=2)
text(0.85, 0.16, "n = 515")
text(1, 0.11, paste(round(renege[2,2], digits=3)), font=2)
text(2, 0.44, "n = 39")
text(2.05, 0.39, paste(round(renege[3,2], digits=3)), font=2)

#dev.off()

###################################################################################################
#	TABLES
###################################################################################################

#	TABLE 1 - Model of factors affecting Service One

y$pctvtage.black <- y$VoteAgeBlack/y$VoteAgeTotal
y$pctvtage.hasian <- y$VoteAgeAsian/y$VoteAgeTotal

lm.out.1 <- lm(Serv1 ~ pwageover55 + pwexperienced + disb + abs + provis + pctvtage.black + pctvtage.hasian +
	 					bg.mdhhinc + alamedaDum + sanmateoDum + pwageintexper + intSMover55 +
						err + paper + bg.pctbaplus + bg.popdens + balk + numpw + line + booths,
						data=y)

summary(lm.out.1)

##########

#	TABLE 2 - Model of factors affecting Service Two

lm.out.2 <- lm(Serv2 ~ alamedaDum + sanmateoDum + ballength + arriv + numpw + booths +
						err + sex + disb + provis + paper + bg.mdfaminc + pctbaplus + 
						pctvtage.black + pctvtage.hasian +	popdens,
						data=y)

summary(lm.out.2)				

### Function for clustering SEs (Arai)
clx <- function(fm, dfcw, cluster){
  library(sandwich)
  library(lmtest)
  M <- length(unique(cluster))
  N <- length(cluster)
  dfc <- (M/(M-1))*((N-1)/(N-fm$rank))
  u <- apply(estfun(fm),2,
        function(x) tapply(x, cluster, sum))
  vcovCL <- dfc*sandwich(fm, meat=crossprod(u)/N)*dfcw
  coeftest(fm, vcovCL) }

lm.1 <- clx(lm.out.1, 1, y$pollst)
lm.2 <- clx(lm.out.2, 1, y$pollst)

###################################################################################################
#	END OF FILE
###################################################################################################